@@ -17,8 +17,6 @@
 
 (define applicable? arity)
 
-;(define sequence? length) -- now built-in for signatures
-
 (define (nth-value n . args) ; untested...
   (and (> (length args) n)
        (list-ref args n)))
@@ -64,15 +62,15 @@
 		    (null? (car x))
 		    (= i (*s7* 'history-size)))
 		(format p "~%error-history:~%    ~S" (car start))
-		(do ((x history (cdr x))
+		(do ((expr history (cdr expr))
 		     (line lines (cdr line))
 		     (f files (cdr f)))
-		    ((null? x))
-		  (format p (if (and (integer? (car line))
-				     (string? (car f))
-				     (not (string=? (car f) "*stdout*")))
-				(values "~%    ~S~40T;~A[~A]" (car x) (car f) (car line))
-				(values "~%    ~S" (car x)))))
+		    ((null? expr))
+		  (if (and (integer? (car line))
+			   (string? (car f))
+			   (not (string=? (car f) "*stdout*")))
+		      (format p "~%    ~S~40T;~A[~A]" (car expr) (car f) (car line))
+		      (format p "~%    ~S" (car expr))))
 		(format p "~%"))
 	     (set! history (cons (car x) history))
 	     (set! lines (cons (and (pair? (car x)) (pair-line-number (car x))) lines))
@@ -84,8 +82,10 @@
 	 (do ((e (outlet ow) (outlet e))) 
 	     ((memq e elist)
 	      (set! (*s7* 'print-length) old-print-length))
-	   (if (> (length e) 0)
-	       (format p "~%~{~A~| ~}~%" e))
+	   (if (and (number? (length e)) ; with-let + mock-data + length method?
+		    (> (length e) 0))
+	       (format p "~%~{~A~| ~}~%" e)
+	       (format p "e: ~S~%" e))
 	   (set! elist (cons e elist))))))))
 
 #|
@@ -138,10 +138,22 @@
 (define (built-in? x) 
   (not (undefined? (eval-string (string-append "#_" (object->string x)))))) ; just a guess...
 
+#|
 (define (the type expr) 
   (if (type expr)
       expr
       (error 'bad-type "~S is ~S but should be ~S" expr (type-of expr) type)))
+|#
+(define (the type expr)
+  (let ((e expr)
+	(bp type))
+    (if (and (procedure? bp)
+	     (signature bp)
+	     (eq? 'boolean? (car (signature bp))))
+	(if (type e)
+	    e
+	    (error 'bad-type "~S is ~S but should be ~S" e (type-of e) bp))
+	(error 'bad-type "~S is not a boolean procedure" bp))))
 
 (define iota 
   (let ((+documentation+ "(iota n (start 0) (incr 1)) returns a list counting from start for n:\n\
@@ -338,7 +350,9 @@
 |#
 
 (define-macro (and-let* vars . body)      ; bind vars, if any is #f stop, else evaluate body with those bindings
-  `(let () (and ,@(map (lambda (v) (cons 'define v)) vars) (begin ,@body))))
+  (if (list? vars)
+      `(let () (and ,@(map (lambda (v) (cons 'define v)) vars) (begin ,@body)))
+      (error 'wrong-type-arg "and-let* var list is ~S" vars)))
 
 (define-macro (let*-temporarily vars . body)
   `(with-let (#_inlet :orig (#_curlet) 
@@ -359,8 +373,6 @@
 			  `(set! (with-let orig ,(car v)) (list-ref saved ,(set! ctr (+ ctr 1)))))))
 		  vars)))))
 
-;;; 14-8-16: moved let-temporarily to s7.c
-
 (define-macro (let/setter vars . body)
   (let ((setters (map (lambda (binding)
 			 (and (pair? (cddr binding))
@@ -889,7 +901,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 	((not (every? integer? ints))
 	 (error 'wrong-type-arg "log-n-of ints arguments, ~A, should all be integers" ints))
 	((not (positive? n))
-	 (error "log-n-of first argument should be positive: ~A" n))
+	 (error 'out-of-range "log-n-of first argument should be positive: ~A" n))
 	(else
 	 (let ((len (length ints)))
 	   (cond ((= len 0) (if (= n 0) -1 0))
@@ -917,17 +929,13 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 (define (byte siz pos) ;; -> cache size, position and mask.
   (list siz pos (ash (- (ash 1 siz) 1) pos)))
 
-(define byte-size car)
-(define byte-position cadr)
-(define byte-mask caddr)
-
 (define (ldb bytespec integer)
-  (ash (logand integer (byte-mask bytespec))
-       (- (byte-position bytespec))))
+  (ash (logand integer (caddr bytespec))
+       (- (cadr bytespec))))
 
 (define (dpb integer bytespec into)
-  (logior (ash (logand integer (- (ash 1 (byte-size bytespec)) 1)) (byte-position bytespec))
-	  (logand into (lognot (byte-mask bytespec)))))
+  (logior (ash (logand integer (- (ash 1 (car bytespec)) 1)) (cadr bytespec))
+	  (logand into (lognot (caddr bytespec)))))
 
 
 ;;; ----------------
@@ -1138,7 +1146,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 
 
 ;;; ----------------
-(define (clamp minimum x maximum)
+(define (clamp minimum x maximum) ; if min>max maybe an error? (clamp 3 2 1) -> 1
   (min maximum (max x minimum)))
 
 (define (1- x) (- x 1))
@@ -1178,6 +1186,8 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
       (if (continuation? ((owlet) 'continue))
 	  (((owlet) 'continue))))))
 
+;;; ----------------
+
 (define (call-with-input-vector v proc)
   (if (vector? v)
       (let ((i -1))
@@ -1187,25 +1197,25 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
       (error 'wrong-type-arg "call-with-input-vector first argument, ~A, should be a vector" v)))
 
 (define (call-with-output-vector proc)
-  (let* ((size 1)
-	 (v (make-vector size #f))
-	 (i 0)
-	 (write-to-vector (lambda (obj p)
-			    (when (= i size) ; make the vector bigger to accommodate the output
-			      (set! v (copy v (make-vector (set! size (* size 2)) #f))))
-			    (set! (v i) obj)
-			    (set! i (+ i 1))
-			    #<unspecified>))) ; that's what write/display return!
-    (proc (openlet
-	   (inlet 'write (lambda* (obj p)
-			   ((if (not (let? p)) write write-to-vector) obj p))
-		  'display (lambda* (obj p)
-			     ((if (not (let? p)) display write-to-vector) obj p))
-		  'format (lambda (p . args)
-			    (if (not (let? p))
-				(apply format p args)
-				(write (apply format #f args) p))))))
-    (subvector v i))) ; ignore extra trailing elements
+  (let ((size 1)
+	(i 0))
+    (let* ((v (make-vector size #f))
+	   (write-to-vector (lambda (obj p)
+			      (when (= i size) ; make the vector bigger to accommodate the output
+				(set! v (copy v (make-vector (set! size (* size 2)) #f))))
+			      (set! (v i) obj)
+			      (set! i (+ i 1))
+			      #<unspecified>))) ; that's what write/display return!
+      (proc (openlet
+	     (inlet 'write (lambda* (obj p)
+			     ((if (not (let? p)) write write-to-vector) obj p))
+		    'display (lambda* (obj p)
+			       ((if (not (let? p)) display write-to-vector) obj p))
+		    'format (lambda (p . args)
+			      (if (not (let? p))
+				  (apply format p args)
+				  (write (apply format #f args) p))))))
+      (subvector v i)))) ; ignore extra trailing elements (i = subvector length)
 
 
 
@@ -1266,12 +1276,6 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 ;; and (let ((a 1) (b 2)) (list (set! a 3) (reflective-probe) b)) -> '(3 () 2)
 ;; or use it to print function args: (define (f a b) (reflective-probe) (+ a b))
 
-;; could this use reactive-lambda* to show changes as well?
-
-
-;;; ----------------
-;;; 28-Jul-18 I've started to rewrite the reactive* code: reactive.scm.
-;;; ----------------
 
 #|
 ;; this tests a bacro for independence of any runtime names
@@ -1376,21 +1380,6 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 	       ((= i new-len) lst)
 	     (set! (lst i) (obj (+ i start))))))))
 
-
-(define (sequence->string val)
-  (let ((ctrl-str (cond ((or (not (sequence? val)) 
-			     (empty? val))    "~S")
-			((int-vector? val)    "#i(~{~D~| ~})")
-			((byte-vector? val)   "#u(~{~D~| ~})")
-			((float-vector? val)  "#r(~{~A~| ~})")
-			((vector? val)        "#(~{~A~| ~})")
-			((let? val)           "(inlet ~{'~A~| ~})")
-			((hash-table? val)    "~W")
-			((not (string? val))  "(~{~A~| ~})")
-			(else                 "\"~{~A~|~}\""))))
-    (format #f ctrl-str val)))
-
-
 ;;; ----------------
 
 (define-macro (elambda args . body)  ; lambda but pass extra arg "*env*" = run-time env
@@ -1408,351 +1397,54 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 
 ;;; ----------------
 
-(if (not (or (defined? 'apropos)
-	     (provided? 'snd)))
-    (define* (apropos name (port *stdout*) (e (rootlet)))
-      (let ((ap-name (if (string? name) 
-			 name 
-			 (if (symbol? name) 
-			     (symbol->string name)
-			     (error 'wrong-type-arg "apropos argument 1 should be a string or a symbol"))))
-	    (ap-env (if (let? e) 
-			e 
-			(error 'wrong-type-arg "apropos argument 3 should be an environment")))
-	    (ap-port (if (output-port? port) 
-			 port
-			 (error 'wrong-type-arg "apropos argument 2 should be an output port"))))
-	(for-each
-	 (lambda (binding)
+(unless (or (defined? 'apropos)
+	    (provided? 'snd))
+  (define* (apropos name (port *stdout*) (e (rootlet)))
+    (let ((ap-name (if (string? name) 
+		       name 
+		       (if (symbol? name) 
+			   (symbol->string name)
+			   (error 'wrong-type-arg "apropos argument 1 should be a string or a symbol"))))
+	  (ap-env (if (let? e) 
+		      e 
+		      (error 'wrong-type-arg "apropos argument 3 should be an environment")))
+	  (ap-port (if (output-port? port) 
+		       port
+		       (error 'wrong-type-arg "apropos argument 2 should be an output port"))))
+      (for-each
+       (lambda (binding)
+	 (if (and (pair? binding)
+		  (string-position ap-name (symbol->string (car binding))))
+	     (format ap-port "~A: ~A~%" 
+		     (car binding) 
+		     (if (procedure? (cdr binding))
+			 (documentation (cdr binding))
+			 (cdr binding)))))
+       ap-env))))
+
+#|
+(define* (apropos->list name (e (rootlet)))
+  (let ((ap-name (if (string? name) 
+		     name 
+		     (if (symbol? name) 
+			 (symbol->string name)
+			 (error 'wrong-type-arg "apropos->let argument 1 should be a string or a symbol")))))
+    (map (lambda (binding)
 	   (if (and (pair? binding)
 		    (string-position ap-name (symbol->string (car binding))))
-	       (format ap-port "~A: ~A~%" 
-		       (car binding) 
-		       (if (procedure? (cdr binding))
-			   (documentation (cdr binding))
-			   (cdr binding)))))
-	 ap-env))))
-
-
-;;; ----------------
-
-;; these need to be globally accessible since they're inserted in arbitrary source
-(define Display #f)
-(define Display-port #f)
-
-(let ((spaces 0)
-      (*display-spacing* 2)         ; (set! ((funclet Display) '*display-spacing*) 1) etc
-      (*display-print-length* 6)
-      (*display* *stderr*)          ; exported via Display-port
-      (e (gensym))
-      (result (gensym))
-      (vlp (gensym)))
-  
-  ;; local symbol access -- this does not affect any other uses of these symbols
-  (set! (setter '*display-spacing* (curlet))
-	(lambda (s v) (if (and (integer? v) (not (negative? v))) v *display-spacing*)))
-  
-  (set! (setter '*display-print-length* (curlet))
-	(lambda (s v) (if (and (integer? v) (not (negative? v))) v *display-print-length*)))
-  
-  ;; export *display* -- just a convenience
-  (set! Display-port (dilambda
-		      (lambda () *display*)
-		      (lambda (val) (set! *display* val))))
-  
-  (define (prepend-spaces)
-    (format *display* (format #f "~~~DC" spaces) #\space))
-  
-  (define (display-let le e) 
-    (let ((vlp (*s7* 'print-length)))
-      (for-each
-       (lambda (slot)
-	 (unless (or (gensym? (car slot))
-		     (eq? (cdr slot) e))
-	   (set! (*s7* 'print-length) *display-print-length*)
-	   (let ((str (sequence->string (cdr slot))))
-	     (set! (*s7* 'print-length) 30)
-	     (format (Display-port) " :~A ~{~A~|~}" (car slot) str))))
-       le)
-      (set! (*s7* 'print-length) vlp)))
-  
-  (define (last lst)
-    (let ((end (let ((len (length lst)))
-		 (list-tail lst (if (negative? len) (abs len) (- len 1))))))
-      (if (pair? end)
-	  (car end)
-	  end)))
-  
-  (define* (butlast lst (result ()))
-    (if (or (not (pair? lst))
-	    (null? (cdr lst)))
-	(reverse result)
-	(butlast (cdr lst) (cons (car lst) result))))
-  
-  (define* (remove-keys args (lst ()))
-    (if (pair? args)
-	(remove-keys (cdr args) 
-		     (if (or (not (keyword? (car args)))
-			     (eq? (car args) :rest))
-			 (cons (car args) lst)
-			 lst))
-	(if (null? args)
-	    (reverse lst)
-	    (append (reverse lst) args))))
-  
-  (define (proc-walk source)
-    
-    (define (display-format str . args)
-      `(let ((,vlp (*s7* 'print-length)))
-	 (with-let (funclet Display)
-	   (set! (*s7* 'print-length) *display-print-length*)
-	   (prepend-spaces))
-	 (format (Display-port) ,str ,@args)
-	 (set! (*s7* 'print-length) ,vlp)))
-  
-    (define (walk-let-body source)
-      (let ((previous (butlast source))
-	    (end (last source)))
-	`(begin
-	   ,@previous
-	   (let ((,result ,end))
-	     (with-let (funclet Display)
-	       (prepend-spaces))
-	     (format (Display-port) "  ~A~A) -> ~A~%"
-		     ,(if (pair? previous) " ... " "")
-		     ',end
-		     ,result)
-	     ,result))))
-    
-    (if (not (pair? source))
-	source
-	(case (car source)
-	  
-	  ((let let* letrec letrec*)                
-	   ;; show local variables, (let vars . body) -> (let vars print-vars . body)
-	   (if (symbol? (cadr source))           ; named let?
-	       (append 
-		(list (car source)               ; let
-		      (cadr source)              ; name
-		      (caddr source)             ; vars
-		      (display-format "(let ~A (~{~A~| ~})~%" (cadr source) '(outlet (curlet))))
-		(walk-let-body (cdddr source)))      ; body
-	       (append 
-		(list (car source)               ; let
-		      (cadr source)              ; vars
-		      (display-format "(~A (~{~A~| ~})~%" (car source) '(outlet (curlet))))
-		(walk-let-body (cddr source)))))     ; body
-	  
-	  ((or and)
-	   ;; report form that short-circuits the evaluation
-	   (cons (car source)
-		 (let ((ctr -1)
-		       (len (- (length source) 2))
-		       (eob (if (eq? (car source) 'or) 'when 'unless)))
-		   (map (lambda (expr)
-			  (set! ctr (+ ctr 1))
-			  `(let ((,result ,expr))
-			     (,eob ,result
-				   (format (Display-port) "  (~A ~A~A~A) -> ~A~%" 
-					   ',(car source)
-					   ,(if (> ctr 0) " ... " "")
-					   ',expr 
-					   ,(if (< ctr len) " ... " "")
-					   ,result))
-			     ,result))
-			(cdr source)))))
-	  
-	  ((begin with-let with-baffle)
-	   ;; report last form
-	   (let ((previous (butlast (cdr source)))
-		 (end (last source)))
-	     `(,(car source)
-	       ,@previous
-	       (let ((,result ,end))
-		 (format (Display-port) "(~A ~A~A) -> ~A~%"
-			 ',(car source)
-			 ,(if (pair? previous) " ... " "")
-			 ',end
-			 ,result)
-		 ,result))))
-	  
-	  ((when unless)
-	   ;; report expr if body not walked, else report last form of body
-	   (let ((previous (butlast (cddr source)))
-		 (end (last (cddr source))))
-	     `(,(car source) (let ((,result ,(cadr source)))
-			       (,(car source) (not ,result)
-				(format (Display-port) "(~A ~A -> ~A ...)~%"
-					',(car source)
-					',(cadr source)
-					,result))
-			       ,result)
-	       ,@previous
-	       (let ((,result ,end))
-		 (format (Display-port) "(~A ... ~A) -> ~A~%"
-			 ',(car source)
-			 ',end
-			 ,result)
-		 ,result))))
-	  
-	  ((quote)
-	   source)
-	  
-	  ((cond)
-	   ;; report form that satisifies cond
-	   (let ((ctr -1)
-		 (len (- (length source) 2)))
-	     `(cond ,@(map (lambda (clause)
-			     (let ((test (car clause))
-				   (body (cdr clause)))
-			       (set! ctr (+ ctr 1))
-			       (if (eq? (car body) '=>) 
-				   `(,test => (lambda (,result) 
-						(let ((,result (,@(cdr body) ,result)))
-						  (format (Display-port) "  (cond ~A~A~A) -> ~A~%" 
-							  ,(if (> ctr 0) " ... " "")
-							  ',clause
-							  ,(if (< ctr len) " ... " "")
-							  ,result)
-						  ,result)))
-				   `(,test (let ((,result (begin ,@body)))
-					     (format (Display-port) "  (cond ~A~A~A) -> ~A~%" 
-						     ,(if (> ctr 0) " ... " "")
-						     ',clause
-						     ,(if (< ctr len) " ... " "")
-						     ,result)
-					     ,result)))))
-			   (cdr source)))))
-	  
-	  ((case)
-	   ;; as in cond but include selector value in [] and report fall throughs
-	   (let ((ctr -1)
-		 (len (- (length source) 3))
-		 (default (member '(else #t) (cddr source) (lambda (a b) 
-							     (memq (car b) a)))))
-	     `(case ,(cadr source)
-		,@(append 
-		   (map (lambda (clause)
-			  (let ((test (car clause))
-				(body (cdr clause)))
-			    (set! ctr (+ ctr 1))
-			    (if (eq? (car body) '=>)
-				`(,test => (lambda (,result)
-					     (let ((,result (,@(cdr body) ,result)))
-					       (format (Display-port) "  (case [~A] ~A~A~A) -> ~A~%" 
-						       ,(cadr source)
-						       ,(if (> ctr 0) " ... " "")
-						       ',clause
-						       ,(if (< ctr len) " ... " "")
-						       ,result)
-					       ,result)))
-				`(,test (let ((,result (begin ,@body)))
-					  (format (Display-port) "  (case [~A] ~A~A~A) -> ~A~%" 
-						  ,(cadr source)
-						  ,(if (> ctr 0) " ... " "")
-						  ',clause
-						  ,(if (< ctr len) " ... " "")
-						  ,result)
-					  ,result)))))
-			(cddr source))
-		   (if default
-		       ()
-		       `((else 
-			  (format (Display-port) "  (case [~A] falls through~%" ,(cadr source)) 
-			  #<unspecified>)))))))
-	  
-	  ((dynamic-wind)
-	   ;; here we want to ignore the first and last clauses, and report the last of the second
-	   (let ((body (let ((p (caddr source)))
-			 (and (eq? (car p) 'lambda)
-			      (cddr p)))))
-	     (if (not body)
-		 source
-		 (let ((previous (and body (butlast body)))
-		       (end (and body (last body))))
-		   `(dynamic-wind
-			,(cadr source)
-			(lambda ()
-			  ,@previous
-			  (let ((,result ,end))
-			    (format (Display-port) "(dynamic-wind ... ~A) -> ~A~%" ',end ,result)
-			    ,result))
-			,(cadddr source))))))
-	  
-	  (else
-	   (cons (proc-walk (car source)) 
-		 (proc-walk (cdr source)))))))
-
-  
-  (define-macro (Display-1 definition)
-    (if (not (and (pair? definition)
-		  (memq (car definition) '(define define*))
-		  (pair? (cdr definition))
-		  (pair? (cadr definition))))
-
-	;; (Display <anything-else>)
-	(proc-walk definition)                                                    ; (Display (+ x 1)) etc
-	
-	;; (Display (define (f ...) ...)
-	(let ((func (caadr definition))
-	      (args (cdadr definition))
-	      (body (cons 'begin (proc-walk (cddr definition)))))
-	  ;(format *stderr* "~A ~A ~A~%" func args body)
-	  (let* ((no-noise-args (remove-keys args))                               ; omit noise words like :optional
-		 (arg-names (cond ((null? args)
-				   ())
-				  ((proper-list? args)                            ; handle (f x ...), (f (x 1) ...), (f . x), and (f x . z)
-				   (map (lambda (a) 
-					  (if (symbol? a) a (car a)))             ; omit the default values
-					no-noise-args))                                
-				  ((pair? args)
-				   (append (butlast no-noise-args) (list :rest (last args))))
-				  (else (list :rest args))))
-		 (call-args (cond ((null? args)
-				   ())
-				  ((proper-list? args)
-				   (if (memq :rest args)
-				       (append (butlast (butlast no-noise-args))   ; also omit the :rest
-					       (list (list 'apply-values (last args))))
-				       arg-names))                                 ; (... y x)
-				  ((pair? args)
-				   (append (butlast no-noise-args)            ; (... y (apply-values x))
-					   (list (list 'apply-values (last args)))))
-				  (else (list (list 'apply-values args))))))     ; (... (apply-values x))
-	    `(define ,func
-	       (define-macro* ,(cons (gensym) args)                                ; args might be a symbol etc
-		 `((lambda* ,(cons ',e ',arg-names)                                ; prepend added env arg because there might be a rest arg
-		     (let ((,',result '?))
-		       (dynamic-wind
-			   (lambda () #f)
-			   (lambda ()                                              ; when function called, show args and caller
-			     (with-let (funclet Display)                           ; indent
-			       (prepend-spaces)
-			       (set! spaces (+ spaces *display-spacing*)))
-			     (format (Display-port) "(~A" ',',func)                ; show args, ruthlessly abbreviated
-			     (((funclet Display) 'display-let) (outlet (outlet (curlet))) ,',e)
-			     (format (Display-port) ")")
-			     (let ((caller (eval '__func__ ,',e)))                 ; show caller 
-			       (if (not (eq? caller #<undefined>))
-				   (format (Display-port) " ;called from ~A" caller)))
-			     (newline (Display-port))
-			     (set! ,',result ,',body))                             ;   but annotated by proc-walk
-			   (lambda ()                                              ; at the end, show the result
-			     (with-let (funclet Display)
-			       (set! spaces (- spaces *display-spacing*))  ; unindent
-			       (prepend-spaces))
-			     (format (Display-port) "    -> ~S~%" ,',result)))))
-		   (curlet) ,,@call-args)))))))                                    ; pass in the original args and the curlet
-	
-  
-  (set! Display Display-1))                                                        ; make Display-1 globally accessible
-
+	       (if (procedure? (cdr binding))
+		   (cons (car binding) (documentation (cdr binding)))
+		   binding)
+	       (values)))
+	 (if (let? e) 
+	     e 
+	     (error 'wrong-type-arg "apropos argument 2 should be an environment")))))
+|#
 
 
 ;;; --------------------------------------------------------------------------------
 
-
-(define (*s7*->list) 
+(define (*s7*->list) ;(let->list *s7*) but not using keywords
   (list 
    :print-length                  (*s7* 'print-length)
    :safety                        (*s7* 'safety)
@@ -1760,18 +1452,23 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
    :heap-size                     (*s7* 'heap-size)
    :free-heap-size                (*s7* 'free-heap-size)
    :max-heap-size                 (*s7* 'max-heap-size)
+   :max-port-data-size            (*s7* 'max-port-data-size)
    :gc-freed                      (*s7* 'gc-freed)
    :gc-protected-objects          (*s7* 'gc-protected-objects)
    :gc-stats                      (*s7* 'gc-stats)
+   :gc-temps-size                 (*s7* 'gc-temps-size)
+   :gc-resize-heap-fraction       (*s7* 'gc-resize-heap-fraction)
+   :gc-resize-heap-by-4-fraction  (*s7* 'gc-resize-heap-by-4-fraction)
    :max-string-length             (*s7* 'max-string-length)
    :max-list-length               (*s7* 'max-list-length)
    :max-vector-length             (*s7* 'max-vector-length)
    :max-vector-dimensions         (*s7* 'max-vector-dimensions)
    :default-hash-table-length     (*s7* 'default-hash-table-length)
    :initial-string-port-length    (*s7* 'initial-string-port-length)
+   :output-port-data-size         (*s7* 'output-port-data-size)
    :default-rationalize-error     (*s7* 'default-rationalize-error)
    :default-random-state          (*s7* 'default-random-state)
-   :equivalent-float-epsilon   (*s7* 'equivalent-float-epsilon)
+   :equivalent-float-epsilon      (*s7* 'equivalent-float-epsilon)
    :hash-table-float-epsilon      (*s7* 'hash-table-float-epsilon)
    :float-format-precision        (*s7* 'float-format-precision)
    :bignum-precision              (*s7* 'bignum-precision)
@@ -1784,10 +1481,14 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
    :max-stack-size                (*s7* 'max-stack-size)
    :autoloading?                  (*s7* 'autoloading?)
    :undefined-identifier-warnings (*s7* 'undefined-identifier-warnings)
+   :undefined-constant-warnings   (*s7* 'undefined-constant-warnings)
+   :accept-all-keyword-arguments  (*s7* 'accept-all-keyword-arguments)
    :catches                       (*s7* 'catches)
    :history-size                  (*s7* 'history-size)
    :history-enabled               (*s7* 'history-enabled)
-   :history                       (*s7* 'history)))
+   :history                       (*s7* 'history)
+   :most-positive-fixnum          (*s7* 'most-positive-fixnum)
+   :most-negative-fixnum          (*s7* 'most-negative-fixnum)))
 
 
 
@@ -1934,6 +1635,13 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 			 #_getenv #_system #_delete-file #_directory->list #_directory? #_file-exists? #_file-mtime))
 		       )))
     (lambda (code)
+      (if (tree-cyclic? code)
+	  (error 'wrong-type-arg "sandbox argument is circular: ~S~%" code))
+      (if (and (pair? code)
+	       (>= (length code) 10000))
+	  (error 'wrong-type-arg
+		 (let-temporarily (((*s7* 'print-length) 16))
+		   (format #f "sandbox code looks bogus: ~S~%" code))))
       ;; block any change to calling program, or access to files, etc
       (let ((new-code 
 	     (call-with-exit
@@ -1942,7 +1650,8 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 		  (cond ((symbol? tree)
 			 (let ((val (symbol->value tree)))
 			   ;; don't accept any symbol with an accessor
-			   (if (or (setter tree)
+			   (if (or ;(setter val)
+				   (setter tree)
 				   (memq tree '(*s7* unquote abort))
 				   (let? val))  ; not sure about this
 			       (quit #f))
@@ -1964,6 +1673,11 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 			((not (pair? tree))
 			 tree)
 			
+			((eq? 'quote (car tree)) ; tree-cyclic? ignores quoted lists
+			 (if (tree-cyclic? (cdr tree))
+			     (error 'wrong-type-arg "sandbox argument is circular: ~S~%" tree))
+			 tree)
+
 			(else
 			 ;; do we need to check IO ports and set! here?
 			 (cons (walk (car tree))
